from asterisk import *

class AsteriskXMLRPC:
	"""
	$Id: PyAsteriskXMLRPC.py,v 1.2 2004/10/06 05:33:33 ww Exp $

	class PyAsteriskXMLRPC
	
	This exposes functionality of and information about the Asterisk PBX. It
	is a partial abstraction of the PyAsterisk class providing a subset of
	the available functions using XMLRPC. It is intended to be used with the
	SimpleXMLRPCServer or SecureXMLRPCServer classes.

	This file is part of the PyAsterisk distribution which is available at
	
	        http://vox.groovy.net/moin/PyAsterisk

	This program is Free Software, provided WITHOUT WARRANTY and released
	subject to the terms of the GNU GPL published at
	
	        http://www.gnu.org/licenses/gpl.html
		
	and as may be amended from time to time.

	Copyleft (.) 2004 Consultants Ars Informatica S.A.R.F.
	"""
	def __init__(self, server):
		self.__server__ = server
		
	def verbose(self, message):
		"""
		Send debugging or informational message to the asterisk
		console.
		"""
		ast_verbose(message + "\n")

	###
	### channel operations
	###
	def active_channels(self):
		"""
		return the number of currently active channels.
		equivalent to len(channels()) but more efficient.
		"""
		return ast_active_channels()
	
	def hangup(self, channel):
		"""
		Try to hang up the named channel. Raises a KeyError if
		the channel does not exits. Returns 0 on success, -1 on
		failure.
		"""
		chan = ast_get_channel_by_name_locked(channel)
		if not chan:
			return 0
		ast_channel_hangup(chan, AST_SOFTHANGUP_EXPLICIT)
		ast_channel_unlock(chan)
		return 0

	def getvar(self, channel, var):
		"""
		Return a the value of a channel variable. Raises a KeyError if
		the channel does not exist, returns the integer 0 if the variable
		is unset, returns the string representation of the variable if it
		is set.
		"""
		chan = ast_get_channel_by_name_locked(channel)
		if not chan:
			return 0
		val = pbx_builtin_getvar_helper(chan, var)
		ast_channel_unlock(chan)
		return val
	
	def setvar(self, channel, var, value):
		"""
		Set the value of a channel variable. Raises a KeyError if the
		channel does not exist. Value will be coerced to a string. Returns
		the integer 0.
		"""
		chan = ast_get_channel_by_name_locked(channel)
		if not chan:
			return 0
		pbx_builtin_setvar_helper(chan, var, value)
		ast_channel_unlock(chan)
		return 0

	def get_cfwd(self, number):
		"""
		get_cfwd(self, number)

		For interaction with featuregroup-d call forwarding.

		return a tuple, (CFIM, CFBS) or
		(call forward unconditionally, call forward on busy)
		for the given number.
		"""
		dbget = pbx_findapp("DBGet")
		if not dbget: return 0
		chan = ast_channel_alloc(0)
		if not chan: return 0
		pbx_exec(chan, dbget, "CFIM=CFIM/" + number, 1)
		pbx_exec(chan, dbget, "CFBS=CFBS/" + number, 1)
		cfim = pbx_builtin_getvar_helper(chan, "CFIM")
		cfbs = pbx_builtin_getvar_helper(chan, "CFBS")
		ast_channel_free(chan)
		return (cfim, cfbs)
	def set_cfwd(self, number, forward):
		"""
		set_cfwd(self, number, forward)
		
		For interaction with featuregroup-d call forwarding.

		set the call forwarding for number to the given
		(CFIM, CFBS) tuple.
		"""
		dbput = pbx_findapp("DBPut")
		dbdel = pbx_findapp("DBDel")
		if not dbput or not dbdel: return 0
		chan = ast_channel_alloc(0)
		if not chan: return 0
		if forward[0]:
			pbx_exec(chan, dbput, "CFIM/" + number + "=" + forward[0], 1)
		else:
			pbx_exec(chan, dbdel, "CFIM/" + number, 1)
		if forward[1]:
			pbx_exec(chan, dbput, "CFBS/" + number + "=" + forward[1], 1)
		else:
			pbx_exec(chan, dbdel, "CFBS/" + number, 1)
		ast_channel_free(chan)
		return 0
	def originate(self, src, dst, cid, context, account):
		"""
		originate(self, src, dst, cid, context, account)

		src is source number (called first)
		dst is destination number
		cid is caller-id presented
		account is for billing.
		"""
		if cid[0] == "1":
			cid = cid[1:]
		args = [src + "@" + context, context, dst, 1,
			"Unspec <%s>" % cid, account, "FORCECID=" + cid, 60000]
		return ast_originate(*args)
