/*
 * app_python.c -- Embedded Python Interpreter for Asterisk
 *
 * This software is distributed under the terms of the GNU GPL, version 2.0 or
 * later, a copy of which you should have recieved in the source distribution 
 * with this file or which is also available at http://www.fsf.org/licences/gpl.html,
 * with one additional clause:
 *
 * Permission is granted to link this program at runtime with the chan_g729a.so
 * library distributed by Digium Inc. in ftp://ftp.digium.com/pub/asterisk/g729
 * or any other implementation of the proprietary and patent-encumbered pseudo
 * standard known as G.729, G.729A or G.729B.
 *
 */

#include <Python.h>
#include <pthread.h>

#include <asterisk/pbx.h>
#include <asterisk/config.h>
#include <asterisk/channel.h>
#include <asterisk/module.h>
#include <asterisk/logger.h>
#include <asterisk/lock.h>

#include "py_asterisk.h"

#define PyASTERISK_VERSION "$Id: app_python.c,v 1.2 2004/11/18 07:17:10 ww Exp $"

#ifdef ASTERISK_1_0
#define ast_pthread_create pthread_create
#endif

static char tdesc[] = "A Python in the DialPlan";
static char app[] = "Python";
static char synopsis[] = "Execute a python script";
static char descrip[] =
"  Python(args)\n\n"
"This will cause the embedded python interpreter to import the\n"
"execute() method from the 'foo' module which must be in sys.path\n"
"The method is called as foo.execute(chan, args) where args are\n"
"optionally passed in the dialplan as above, and chan is the name\n"
"of the channel that can be used as a key in the PyAsterisk API\n";

static int app_python(struct ast_channel *chan, void *data)
{
  PyInterpreterState *interp;
  PyThreadState *tstate;
  PyObject *module_name, *module, *dict, *func, *chanp, *datap, *arglist;
  PyObject *err;

  struct localuser *u = NULL;
  int res;
  char script[256], *arg;

  /* tell asterisk we are in use */
  res = ResPyAsteriskUserAdd(chan, u);
  if(res)
    return res;

  /* extract the script name from our arguments */
  memset(script, 0, sizeof(script));
  arg = strchr((char *)data, '|');
  if(arg) {
    int len = arg - (char *)data;
    strncpy(script, data, len < sizeof(script) ? len : sizeof(script));
  } else {
    strncpy(script, data, sizeof(script));
  }

  /* get the python interpreter */
  interp = ResPyAsteriskInterpreter();
  if(!interp) {
    ResPyAsteriskUserRemove(chan, u);
    return -1;
  }

  /* make a new thread for us */
  tstate = PyThreadState_New(interp);
  if(!tstate) {
    ResPyAsteriskUserRemove(chan, u);
    return -1;
  }

  /* acquire the thread state */
  PyEval_AcquireThread(tstate); 

  /* get the module name */
  module_name = PyString_FromString(script);
  if(!module_name) {
    ast_log(LOG_ERROR, "Python cannot allocate string\n");
    res = -1;
    goto cleanup;
  }

  /* import the module, after we don't need the nmae anymore */
  module = PyImport_Import(module_name);
  Py_DECREF(module_name);

  if(!module) {
    ast_log(LOG_ERROR, "Unable to import %s\n", script);
    res = -1;
    goto cleanup;
  }

  /* get dictionary of symbols for module -- dict ref is borrowed */
  dict = PyModule_GetDict(module);
  if(!dict) {
    ast_log(LOG_ERROR, "Unable to get module dictionary for %s\n", script);
    Py_DECREF(module);
    res = -1;
    goto cleanup;
  }

  /* look for the execute function -- func ref is borrowed */
  func = PyDict_GetItemString(dict, "execute");
  if(!func || !PyCallable_Check(func)) {
    ast_log(LOG_ERROR, "%s.execute does not exist or is not callable\n", script);
    Py_DECREF(module);
    res = -1;
    goto cleanup;
  }

  /* build the arguments to the function */
  if(arg)
    arglist = PyTuple_New(2);
  else
    arglist = PyTuple_New(1);

  if(!arglist) {
    ast_log(LOG_ERROR, "Cannot allocate arguments\n");
    Py_DECREF(module);
    res = -1;
    goto cleanup;
  }

  chanp = PyAsterisk_BuildChannelObj(chan);
  if(!chanp) {
    ast_log(LOG_ERROR, "Cannot allocate Python channel object\n");
    Py_DECREF(arglist);
    Py_DECREF(module);
    res = -1;
    goto cleanup;
  }
  /* borrows reference to chanp */
  PyTuple_SetItem(arglist, 0, chanp);

  if(arg) {
    /* because of the strchr above, arg should be pointing to the
     * comma in the argument list from the dialplan. eat the !.
     */
    arg++;
    /* eat whitespace */
    while(isspace(*arg))
      arg++;
    datap = PyString_FromString(arg);
    if(!datap) {
      Py_DECREF(arglist);
      Py_DECREF(module);
      res = -1;
      goto cleanup;
    }
    PyTuple_SetItem(arglist, 1, datap);
  }
  
  /* finally, run the function */
  PyObject_CallObject(func, arglist);

  err = PyErr_Occurred();
  if(err) {
    PyErr_Print();
    res = -1;
  }

  Py_DECREF(arglist);
  Py_DECREF(module);

cleanup:
  PyEval_ReleaseThread(tstate);
  PyThreadState_Delete(tstate);

  ResPyAsteriskUserRemove(chan, u);
  return res;
}

/*
 * Asterisk Module Housekeeping. Ver' simple.
 */
  
STANDARD_LOCAL_USER;
LOCAL_USER_DECL;

int load_module(void)
{
  int res;
  res = ast_register_application(app, app_python, synopsis, descrip);
  return 0;
}

int unload_module(void)
{
  STANDARD_HANGUP_LOCALUSERS;
  return ast_unregister_application(app);
}

char *description(void)
{
  return tdesc;
}

int usecount(void)
{
  int res;
  STANDARD_USECOUNT(res);
  return res;
}

char *key()
{
  return ASTERISK_GPL_KEY;
}
