/*
 * cdr_python.c -- Embedded Python Interpreter for Asterisk
 *                 Call Detail Record backend
 *
 * This software is distributed under the terms of the GNU GPL, version 2.0 or
 * later, a copy of which you should have recieved in the source distribution 
 * with this file or which is also available at http://www.fsf.org/licences/gpl.html,
 * with one additional clause:
 *
 * Permission is granted to link this program at runtime with the chan_g729a.so
 * library distributed by Digium Inc. in ftp://ftp.digium.com/pub/asterisk/g729
 * or any other implementation of the proprietary and patent-encumbered pseudo
 * standard known as G.729, G.729A or G.729B.
 *
 */

#include <Python.h>
#include <pthread.h>

#include <sys/types.h>
#include <asterisk/channel.h>
#include <asterisk/cdr.h>
#include <asterisk/module.h>
#include <asterisk/logger.h>
#include <asterisk/utils.h>

#include "py_asterisk.h"

#define PyASTERISK_CDR_VERSION "$Id: cdr_python.c,v 1.1 2004/11/18 07:17:10 ww Exp $"

static char *desc = "Python CDR Backend";
static char *name = "cdr_python";

static PyObject *cdr_func = NULL;

static int cdr_python(struct ast_cdr *cdr)
{
  PyInterpreterState *interp;
  PyThreadState *tstate;
  PyObject *cdrobj, *args;
  PyObject *err;
  int res = 0;

  if(!cdr_func)
    return -1;

  interp = ResPyAsteriskInterpreter();
  if(!interp)
    return -1;

  tstate = PyThreadState_New(interp);
  if(!tstate)
    return -1;

  PyEval_AcquireThread(tstate);

  cdrobj = PyAsterisk_BuildCdrObj(cdr);
  if(!cdrobj) {
    ast_log(LOG_ERROR, "Could not build CDR object\n");
    res = -1;
    goto cleanup;
  }

  args = PyTuple_New(1);
  if(!args) {
    ast_log(LOG_ERROR, "Could not build tuple\n");
    res = -1;
    Py_DECREF(cdrobj);
    goto cleanup;
  }
  PyTuple_SetItem(args, 0, cdrobj);

  PyObject_CallObject(cdr_func, args);

  err = PyErr_Occurred();
  if(err) {
    res = -1;
    PyErr_Print();
  }

  Py_DECREF(args);

 cleanup:

  PyEval_ReleaseThread(tstate);
  PyThreadState_Delete(tstate);

  return res;
}

char *description(void)
{
  return desc;
}

int unload_module(void)
{
  if(cdr_func) {
    Py_DECREF(cdr_func);
    cdr_func = NULL;
  }

  ast_cdr_unregister(name);
  return 0;
}

int load_module(void)
{
  PyInterpreterState *interp;
  PyThreadState *tstate;
  PyObject *module_name, *module, *dict;
  int res = 0;

  ast_verbose("Loading Python CDR backend\n");

  /* acquire python thread state so that we can get
     a reference to the CDR processing function */
  interp = ResPyAsteriskInterpreter();
  if(!interp)
    return -1;

  tstate = PyThreadState_New(interp);
  if(!tstate)
    return -1;

  PyEval_AcquireThread(tstate);

  module_name = PyString_FromString(PyASTERISK_CONFIG_SCRIPT);
  if(!module_name) {
    res = -1;
    goto cleanup;
  }

  /* import hteh PyAsteriskConfig module */
  module = PyImport_Import(module_name);
  Py_DECREF(module_name);

  if(!module) {
    ast_log(LOG_ERROR, "Unable to import " PyASTERISK_CONFIG_SCRIPT " for CDR backend\n");
    res = -1;
    goto cleanup;
  }

  /* look in the module's dictionary for the cdr function */
  dict = PyModule_GetDict(module);
  if(!dict) {
    ast_log(LOG_ERROR, "Unable to get dict for module " PyASTERISK_CONFIG_SCRIPT "\n");
    Py_DECREF(module);
    res = -1;
    goto cleanup;
  }

  /* dict is a borrowed reference */
  cdr_func = PyDict_GetItemString(dict, "cdr");

  if(!cdr_func || !PyCallable_Check(cdr_func)) {
    ast_log(LOG_ERROR, PyASTERISK_CONFIG_SCRIPT ".cdr is not callable\n");
    Py_DECREF(module);
    res = -1;
    goto cleanup;
  }

  /* all is well, we do not need the module reference anymore,
     but we do need a reference to the function pointer */
  Py_INCREF(cdr_func);
  Py_DECREF(module);

  /* register our CDR handler with asterisk */
  res = ast_cdr_register(name, desc, cdr_python);
  if(res) {
    ast_log(LOG_ERROR, "Unable to register Python CDR handling\n");
    Py_DECREF(cdr_func);
    cdr_func = NULL;
    res = -1;
  }

 cleanup:
  PyEval_ReleaseThread(tstate);
  PyThreadState_Delete(tstate);
  
  return res;
}

int reload(void)
{
  return 0;
}

int usecount(void)
{
  return 0;
}

char *key()
{
  return ASTERISK_GPL_KEY;
}
