/*
 * res_python.c -- Embedded Python Interpreter for Asterisk
 *
 * This software is distributed under the terms of the GNU GPL, version 2.0 or
 * later, a copy of which you should have recieved in the source distribution 
 * with this file or which is also available at http://www.fsf.org/licences/gpl.html,
 * with one additional clause:
 *
 * Permission is granted to link this program at runtime with the chan_g729a.so
 * library distributed by Digium Inc. in ftp://ftp.digium.com/pub/asterisk/g729
 * or any other implementation of the proprietary and patent-encumbered pseudo
 * standard known as G.729, G.729A or G.729B.
 *
 */

#include <Python.h>
#include <pthread.h>

#include <asterisk/pbx.h>
#include <asterisk/config.h>
#include <asterisk/channel.h>
#include <asterisk/module.h>
#include <asterisk/logger.h>
#include <asterisk/lock.h>

#include "py_asterisk.h"

#define PyASTERISK_VERSION "$Id: res_python.c,v 1.6 2004/11/18 07:17:10 ww Exp $"

#ifdef ASTERISK_1_0
#define ast_pthread_create pthread_create
#endif

static char tdesc[] = "Asterisk Embedded Python Interpreter";
static pthread_t interp_thread;
static PyInterpreterState *python_interp = NULL;

STANDARD_LOCAL_USER;
LOCAL_USER_DECL;

int ResPyAsteriskUserAdd(struct ast_channel *chan, struct localuser *u)
{
  LOCAL_USER_ADD(u);
  return 0;
}
int ResPyAsteriskUserRemove(struct ast_channel *chan, struct localuser *u)
{
  LOCAL_USER_REMOVE(u);
  return 0;
}

PyInterpreterState *ResPyAsteriskInterpreter(void)
{
  return python_interp;
}

static PyObject *PyAsteriskLog(PyObject *self, PyObject *args)
{
  int level;
  const char *s;
  if(!PyArg_ParseTuple(args, "is", &level, &s))
    return NULL;
  ast_log(level, _A_, "%s\n", s);
  Py_INCREF(Py_None);
  return Py_None;
}

/*
 * Asterisk Module Housekeeping. Ver' simple.
 */

static void *python_init(void *dummy)
	/* ARGUSED */
{
  PyThreadState *tstate;

  ast_log(LOG_DEBUG, "Initializing Python Interpreter.\n");
  ast_verbose("Python %s %s\n", Py_GetPlatform(), Py_GetVersion());

  Py_SetProgramName(PyASTERISK_NAME);
  PyEval_InitThreads();
  Py_Initialize();
  init_asterisk();

  /* Save the main thread... */
  tstate = PyEval_SaveThread();
  python_interp = tstate->interp;
  PyEval_RestoreThread(tstate);

  /* Run our intialization function, which
   * should not return 
   */
  PyRun_SimpleString("import sys\n"
		     "sys.path.append('" PyASTERISK_LIB_DIR "')\n"
		     "from " PyASTERISK_CONFIG_SCRIPT " import start\n"
		     "try:\n"
		     "    start()\n"
		     "except:\n"
		     "    from traceback import format_exception\n"
		     "    from sys import exc_info\n"
		     "    from string import join\n"
		     "    from _asterisk import ast_verbose\n"
		     "    ast_verbose(join(apply(format_exception, exc_info()), '\\n'))\n");

  ast_log(LOG_DEBUG, "Main Python thread returned");

  return NULL;
}

int unload_module(void)
{
  PyThreadState *tstate;

  STANDARD_HANGUP_LOCALUSERS;

  ast_log(LOG_DEBUG, "Terminating Python Interpreter.\n");

  if(python_interp) {
    /* interp is your reference to an interpreter object. */
    tstate = PyThreadState_New(python_interp);
    PyEval_AcquireThread(tstate);

    PyRun_SimpleString("from " PyASTERISK_CONFIG_SCRIPT " import stop\n"
		       "try:\n"
		       "    stop()\n"
		       "except:\n"
		       "    from traceback import format_exception\n"
		       "    from sys import exc_info\n"
		       "    from string import join\n"
		       "    from _asterisk import ast_verbose\n"
		       "    ast_verbose(join(apply(format_exception, exc_info()), '\\n'))\n");
  
    /* Release the thread. No Python API allowed beyond this point. */
    PyEval_ReleaseThread(tstate);
    PyThreadState_Delete(tstate);
  }

  Py_Finalize();
  python_interp = NULL;

  ast_log(LOG_DEBUG, "res_python.c unloaded.\n");
  return 0;
}

int load_module(void)
{
  ast_verbose(PyASTERISK_NAME " " PyASTERISK_VERSION "\n"
	      PyASTERISK_COPYRIGHT "\n");

  if(ast_pthread_create(&interp_thread, NULL, python_init, NULL)) {
    ast_log(LOG_ERROR, "Unable to create python interpreter\n");
    return -1;
  }
  ast_log(LOG_DEBUG, "res_python.c loaded.\n");
  return 0;
}

char *description(void)
{
        return tdesc;
}

int usecount(void)
{
        int res;
        STANDARD_USECOUNT(res);
        return res;
}

char *key()
{
        return ASTERISK_GPL_KEY;
}
